//
//  tank
//
//  Import and Playlist Daemon for autoradio project
//
//
//  Copyright (C) 2017 Christian Pointner <equinox@helsinki.at>
//
//  This file is part of tank.
//
//  tank is free software: you can redistribute it and/or modify
//  it under the terms of the GNU General Public License as published by
//  the Free Software Foundation, either version 3 of the License, or
//  any later version.
//
//  tank is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//  GNU General Public License for more details.
//
//  You should have received a copy of the GNU General Public License
//  along with tank. If not, see <http://www.gnu.org/licenses/>.
//

package store

import (
	"encoding/json"
	"errors"

	"github.com/coreos/bbolt"
)

func getFilesBucket(tx *bolt.Tx, group string) (files *bolt.Bucket, err error) {
	var gb *bolt.Bucket
	if gb, err = getGroupBucket(tx, group); err != nil {
		return
	}
	if gb == nil {
		if !tx.Writable() {
			return // for read-only transactions this function might return nil for files
		}
		if gb, err = createGroup(tx, group); err != nil {
			return
		}
	}
	files = gb.Bucket([]byte(filesBn))
	if files == nil {
		err = errors.New("invalid store: files bucket of group '" + group + "' not found")
	}
	return
}

func (st *Store) ListFiles(group string) (files Files, err error) {
	err = st.db.View(func(tx *bolt.Tx) error {
		fb, err := getFilesBucket(tx, group)
		if err != nil {
			return err
		}
		if fb == nil {
			return nil
		}
		files = make(Files)
		err = fb.ForEach(func(k, v []byte) error {
			var file File
			if err := json.Unmarshal(v, &file); err != nil {
				return err
			}
			files[string(k)] = file
			return nil
		})
		return err
	})
	return
}

func (st *Store) CreateFile(group string, file File) (id string, err error) {
	err = st.db.Update(func(tx *bolt.Tx) error {
		fb, err := getFilesBucket(tx, group)
		if err != nil {
			return err
		}
		id = "1124235" // TODO: create id using bolt sequences
		v, err := json.Marshal(file)
		if err != nil {
			return err
		}
		return fb.Put([]byte(id), v)
	})
	return
}

func (st *Store) GetFile(group, id string) (File, error) {
	// TODO: implement this
	return File{}, ErrNotImplented
}

func (st *Store) UpdateFile(group, id string, new File) (File, error) {
	// TODO: implement this
	return File{}, ErrNotImplented
}

func (st *Store) DeleteFile(group, id string) error {
	// TODO: implement this
	return ErrNotImplented
}
